@@ -1,11 +1,9 @@
-#include "../include/features.h"
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <sys/time.h>
 #include <sys/types.h>
+#include <sys/time.h>
 #include <sys/select.h>
 #include <sys/socket.h>
 
@@ -38,7 +36,7 @@
 #endif
 
 #ifdef __CYGWIN__
-#include "../include/cygwin.h"
+#include "cygwin.h"
 #endif
 
 #include <netdb.h>
@@ -52,18 +50,15 @@
 static const size_t   MAX_IPV4_HDR_SIZE  = 60;
 static const uint32_t CALIBRATION_CYCLES = 100,
                       PKT_BURST_SCALE    = 1000;
-static const int64_t  MAX_PKT_TIME_SEC   = 8589934592,
-                      MAX_PKT_TIME_NSEC  = 999999999;
 
 static char *prog_name;
 
 static void get_time(struct timespec *ts)
 {
-#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME)
 #if defined(CLOCK_HIGHRES)
-    const clockid_t id = CLOCK_HIGHRES;
+    const clockid_t id = CLOCK_HIGHRES; /* Solaris */
 #elif defined(CLOCK_MONOTONIC_RAW)
-    const clockid_t id = CLOCK_MONOTONIC_RAW;
+    const clockid_t id = CLOCK_MONOTONIC_RAW; /* Linux */
 #elif defined(CLOCK_MONOTONIC)
     const clockid_t id = CLOCK_MONOTONIC;
 #else
@@ -76,24 +71,44 @@ static void get_time(struct timespec *ts)
         ts->tv_sec  = 0;
         ts->tv_nsec = 0;
     }
-#else /* HAVE_CLOCK_GETTIME && CLOCK_REALTIME */
-    struct timeval tv;
+}
 
-    if (gettimeofday(&tv, NULL) < 0) {
-        fprintf(stderr, "%s: gettimeofday() failed: %s\n", prog_name, strerror(errno));
-
-        ts->tv_sec  = 0;
-        ts->tv_nsec = 0;
+static int64_t int64_sub(int64_t i1, int64_t i2)
+{
+    if (i2 < 0) {
+        if (i1 > INT64_MAX + i2) {
+            return INT64_MAX;
+        }
     } else {
-        ts->tv_sec  = tv.tv_sec;
-        ts->tv_nsec = tv.tv_usec * 1000;
+        if (i1 < INT64_MIN + i2) {
+            return INT64_MIN;
+        }
     }
-#endif /* HAVE_CLOCK_GETTIME && CLOCK_REALTIME */
+
+    return i1 - i2;
 }
 
 static int64_t ts_sub(const struct timespec *ts1, const struct timespec *ts2)
 {
-    return ((int64_t)ts1->tv_sec - (int64_t)ts2->tv_sec) * 1000000 + (ts1->tv_nsec - ts2->tv_nsec) / 1000;
+    int64_t sec_diff = int64_sub(ts1->tv_sec, ts2->tv_sec);
+
+    if (sec_diff > INT32_MAX) {
+        sec_diff = INT32_MAX;
+    }
+    if (sec_diff < INT32_MIN) {
+        sec_diff = INT32_MIN;
+    }
+
+    int64_t nsec_diff = int64_sub(ts1->tv_nsec, ts2->tv_nsec);
+
+    if (nsec_diff > INT32_MAX) {
+        nsec_diff = INT32_MAX;
+    }
+    if (nsec_diff < INT32_MIN) {
+        nsec_diff = INT32_MIN;
+    }
+
+    return sec_diff * 1000000 + nsec_diff / 1000;
 }
 
 static uint16_t cksum(const char *data, size_t size)
@@ -243,7 +258,7 @@ static void sendmmsg_ping(bool ipv4_mode, int sock, const struct addrinfo *to_ai
         if (res < 0) {
             fprintf(stderr, "%s: sendmmsg() failed: %s\n", prog_name, strerror(errno));
         } else if ((unsigned int)res != vlen) {
-            fprintf(stderr, "%s: sendmmsg() warning: packets to send: %u, sent: %d\n", prog_name, vlen, res);
+            fprintf(stderr, "%s: sendmmsg() packets to send: %u, sent: %d\n", prog_name, vlen, res);
         }
     }
 }
@@ -268,7 +283,7 @@ static void send_ping(bool ipv4_mode, int sock, const struct addrinfo *to_ai, si
     if (res < 0) {
         fprintf(stderr, "%s: sendto() failed: %s\n", prog_name, strerror(errno));
     } else if ((size_t)res != pkt_size) {
-        fprintf(stderr, "%s: sendto() warning: packet size: %zu, sent: %zd\n", prog_name, pkt_size, res);
+        fprintf(stderr, "%s: sendto() packet size: %zu, sent: %zd\n", prog_name, pkt_size, res);
     }
 }
 
@@ -300,26 +315,21 @@ static void process_ping4(const char *packet, ssize_t pkt_size, uint16_t ident,
                         memcpy(&pkt_time, &packet[hdr_len + sizeof(icmp4)], sizeof(pkt_time));
 
                         if (pkt_time.tv_sec != 0 || pkt_time.tv_nsec != 0) {
-                            if ((int64_t)pkt_time.tv_sec  >= 0 && (int64_t)pkt_time.tv_sec  <= MAX_PKT_TIME_SEC &&
-                                         pkt_time.tv_nsec >= 0 &&          pkt_time.tv_nsec <= MAX_PKT_TIME_NSEC) {
-                                struct timespec now;
+                            struct timespec now;
 
-                                get_time(&now);
+                            get_time(&now);
 
-                                int64_t rtt = ts_sub(&now, &pkt_time) / 1000;
+                            int64_t rtt = ts_sub(&now, &pkt_time) / 1000;
 
-                                if (rtt >= 0) {
-                                    if (*min_rtt > (uint64_t)rtt) {
-                                        *min_rtt = rtt;
-                                    }
-                                    if (*max_rtt < (uint64_t)rtt) {
-                                        *max_rtt = rtt;
-                                    }
-                                    if (*received_count > 0) {
-                                        *average_rtt = (*average_rtt * (*received_count - 1) + rtt) / *received_count;
-                                    }
-                                } else {
-                                    fprintf(stderr, "%s: packet has an invalid timestamp\n", prog_name);
+                            if (rtt >= 0) {
+                                if (*min_rtt > (uint64_t)rtt) {
+                                    *min_rtt = rtt;
+                                }
+                                if (*max_rtt < (uint64_t)rtt) {
+                                    *max_rtt = rtt;
+                                }
+                                if (*received_count > 0) {
+                                    *average_rtt = (*average_rtt * (*received_count - 1) + rtt) / *received_count;
                                 }
                             } else {
                                 fprintf(stderr, "%s: packet has an invalid timestamp\n", prog_name);
@@ -350,26 +360,21 @@ static void process_ping6(const char *packet, ssize_t pkt_size, uint16_t ident,
                 memcpy(&pkt_time, &packet[sizeof(icmp6)], sizeof(pkt_time));
 
                 if (pkt_time.tv_sec != 0 || pkt_time.tv_nsec != 0) {
-                    if ((int64_t)pkt_time.tv_sec  >= 0 && (int64_t)pkt_time.tv_sec  <= MAX_PKT_TIME_SEC &&
-                                 pkt_time.tv_nsec >= 0 &&          pkt_time.tv_nsec <= MAX_PKT_TIME_NSEC) {
-                        struct timespec now;
+                    struct timespec now;
 
-                        get_time(&now);
+                    get_time(&now);
 
-                        int64_t rtt = ts_sub(&now, &pkt_time) / 1000;
+                    int64_t rtt = ts_sub(&now, &pkt_time) / 1000;
 
-                        if (rtt >= 0) {
-                            if (*min_rtt > (uint64_t)rtt) {
-                                *min_rtt = rtt;
-                            }
-                            if (*max_rtt < (uint64_t)rtt) {
-                                *max_rtt = rtt;
-                            }
-                            if (*received_count > 0) {
-                                *average_rtt = (*average_rtt * (*received_count - 1) + rtt) / *received_count;
-                            }
-                        } else {
-                            fprintf(stderr, "%s: packet has an invalid timestamp\n", prog_name);
+                    if (rtt >= 0) {
+                        if (*min_rtt > (uint64_t)rtt) {
+                            *min_rtt = rtt;
+                        }
+                        if (*max_rtt < (uint64_t)rtt) {
+                            *max_rtt = rtt;
+                        }
+                        if (*received_count > 0) {
+                            *average_rtt = (*average_rtt * (*received_count - 1) + rtt) / *received_count;
                         }
                     } else {
                         fprintf(stderr, "%s: packet has an invalid timestamp\n", prog_name);
@@ -571,6 +576,9 @@ int main(int argc, char *argv[])
     }
 
     if (exit_val != EX_OK) {
+#if defined(PACKAGE_STRING)
+        fprintf(stderr, "%s is part of the %s package\n", prog_name, PACKAGE_STRING);
+#endif
         fprintf(stderr, "Usage: %s [-4 | -6] [-u buf_size] [-r reporting_period] [-T tos(v4) | traf_class(v6)] [-B bind_addr]"
                         " -b kbps -s pkt_size -v volume target\n", prog_name);
 
